@@ -34,9 +34,10 @@ if(getRversion() >= "2.15.1") {
 #' @param ncores The number of cores to use for parallel computing
 #' @param input_out A vector of variables to be returned in the output data frame
 #' @param ipd Integer taking value 0 if no IPD data returned, 1 for full IPD data returned, and 2 IPD data but aggregating events
+#' @param constrained Boolean, FALSE by default, which runs the simulation with patients not interacting with each other, TRUE if resources are shared within an arm (allows constrained resources)
 #' @param timed_freq If NULL, it does not produce any timed outputs. Otherwise should be a number (e.g., every 1 year)
 #' @param debug If TRUE, will generate a log file
-#' @param accum_backwards If TRUE, the ongoing accumulators will count backwards (i.e., the current value is applied until the previous update). If FALSE, the current value is applied between the current event and the next time it is updated. If TRUE, user must use `modify_item` and `modify_item_seq` or results will be incorrect.
+#' @param accum_backwards If TRUE, the ongoing accumulators will count backwards (i.e., the current value is applied until the previous update). If FALSE, the current value is applied between the current event and the next time it is updated.
 #' @param continue_on_error If TRUE, on error  at patient stage will attempt to continue to the next simulation (only works if n_sim and/or n_sensitivity are > 1, not at the patient level)
 #' @param seed Starting seed to be used for the whole analysis. If null, it's set to 1 by default.
 #'
@@ -49,7 +50,7 @@ if(getRversion() >= "2.15.1") {
 #' @importFrom progressr handlers
 #' @importFrom progressr progressor
 #' @importFrom progressr handler_txtprogressbar
-#' @importFrom rlang env_clone
+#' @importFrom stats setNames
 #'
 #' @export
 #' 
@@ -78,9 +79,7 @@ if(getRversion() >= "2.15.1") {
 #' so we want to make sure to add o_q = utility at event 3 before updating utility. The program will automatically 
 #' look back until event 1). Note that in previous versions of the package backward was the default, and now this has switched to forward.
 #' 
-#' If using `accum_backwards = TRUE`, then it is mandatory for the user to use `modify_item` and `modify_item_seq` in event reactions,
-#'   as the standard assignment approach (e.g., `a <- 5`) will not calculate the right results, particularly in the presence of
-#'   conditional statements.
+#'  The requirement to use `modify_item` if using `accum_backwards = TRUE`, is no longer the case thanks to a new method using active bindings, so it can be used normally.
 #' 
 #'  If the `cycle` lists are used, then it is expected the user will declare as well the name of the variable
 #'   pasted with `cycle_l` and `cycle_starttime` (e.g., c_default_cycle_l and c_default_cycle_starttime) to 
@@ -91,7 +90,7 @@ if(getRversion() >= "2.15.1") {
 #'  `debug = TRUE` will export a log file with the timestamp up the error in the main working directory. Note that
 #'  using this mode without modify_item or modify_item_seq may lead to inaccuracies if assignments are done in non-standard ways,
 #'  as the AST may not catch all the relevant assignments (e.g., an assigment like assign(paste("x_",i),5)
-#'   in a loop will not be identified, unless using modify_item()).
+#'   in a loop will not be identified).
 #'   
 #'   If `continue_on_error` is set to FALSE, it will only export analysis level inputs due to the parallel engine
 #'    (use single-engine for those inputs) `continue_on_error` will skip the current simulation 
@@ -133,15 +132,15 @@ if(getRversion() >= "2.15.1") {
 #'              input = {}) %>%
 #'   add_reactevt(name_evt = "sicker",
 #'                input = {
-#'                  modify_item(list(q_default = util.sicker,
-#'                                   c_default = cost.sicker + if(arm=="int"){cost.int}else{0},
-#'                                   fl.sick = 0)) 
+#'                  q_default <- util.sicker
+#'                  c_default <- cost.sicker + if(arm=="int"){cost.int}else{0}
+#'                  fl.sick <- 0
 #'                }) %>%
 #'   add_reactevt(name_evt = "death",
 #'                input = {
-#'                  modify_item(list(q_default = 0,
-#'                                   c_default = 0, 
-#'                                   curtime = Inf)) 
+#'                  q_default <- 0
+#'                  c_default <- 0
+#'                  curtime <- Inf
 #'                }) 
 #'                
 #' util_ongoing <- "q_default"
@@ -162,7 +161,6 @@ if(getRversion() >= "2.15.1") {
 #' ipd = 1,
 #' ncores = 1)
 #' 
-
 run_sim_parallel <- function(arm_list=c("int","noint"),
                              sensitivity_inputs=NULL,
                              common_all_inputs=NULL,
@@ -187,6 +185,7 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
                              ncores=1,
                              input_out = character(),
                              ipd = 1,
+                             constrained = FALSE,
                              timed_freq = NULL,
                              debug = FALSE,
                              accum_backwards = FALSE,
@@ -194,12 +193,41 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
                              seed = NULL){
   
   
+  # # ---- Error-context beacon (for clean messages; keeps base traceback) ----
+  .warden_ctx <- new.env(parent = emptyenv())
+  .warden_ctx$last <- NULL
+  .set_last_ctx(stage="Error in setup:initial code setup", .warden_ctx = .warden_ctx)
+  
+  log_sink <- new.env(parent = emptyenv())
+  log_sink$entries <- list()
+  
+  on.exit({
+    do.call(RNGkind, as.list(rng_kind_store))
+    if (debug) export_log(log_sink$entries,paste0("log_model_",format(Sys.time(), "%Y_%m_%d_%Hh_%mm_%Ss"),".txt"))
+    ctx <- .warden_ctx$last
+    
+    message(
+      paste0(ctx$stage,"; ",
+             ifelse(!is.na(ctx$sens),paste0("Analysis ", ctx$sens,"; "),""),
+             ifelse(!is.na(ctx$simulation),paste0("Simulation ", ctx$simulation,"; "),""),
+             ifelse(!is.na(ctx$arm),paste0("Arm ", ctx$arm,"; "),""),
+             ifelse(!is.na(ctx$patient_id),paste0("Patient ", ctx$patient_id,"; "),""),
+             ifelse(!is.na(ctx$event),paste0("Event ", ctx$event,"; "),""),
+             ifelse(!is.na(ctx$time),paste0("Time ", ctx$time,"; "),"")
+      ))
+    
+    if(ctx$stage!="Simulation finalized"){message("An error occurred somewhere. Traceback available through traceback()")}
+    
+  }, add=TRUE)
+  
+  
   # Set-up basics -----------------------------------------------------------
   
   #Store original rng kind and use L'Ecuyer-CMRG 
-  rng_kind_store <- RNGkind()[1]
+  rng_kind_store <- RNGkind()
   RNGkind("L'Ecuyer-CMRG")
   
+  
   #Stop simulation if forbidden objects are used.
   list_forbidden_names <-  c("arm", "arm_list", "categories_for_export", "cur_evtlist", "curtime", "evt", "i", "prevtime", "sens", "simulation", "sens_name_used","list_env","uc_lists","npats","ipd") 
   matched_list_forbidden <- which(list_forbidden_names %in% c(
@@ -255,13 +283,7 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
       if(!is.null(categories_other_ongoing)){categories_other_ongoing},
       if(!is.null(categories_other_instant)){categories_other_instant}
     ))
-  
-  
-  env_setup_sens <- is.language(sensitivity_inputs)
-  env_setup_sim <- is.language(common_all_inputs)
-  env_setup_pt <- is.language(common_pt_inputs)
-  env_setup_arm <- is.language(unique_pt_inputs)
-  
+
   output_sim <- list()
   
   final_log <- list()
@@ -279,15 +301,15 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
   }
   
   progressr::handlers(progressr::handler_txtprogressbar(width=100))
-  
+  options(progressr.interrupts = FALSE)
   progressr::with_progress({
     pb <- progressr::progressor(min(npats*length_sensitivities*n_sim,50)) 
     
   for (sens in 1:length_sensitivities) {
-    message(paste0("Analysis number: ",sens))
+    .skip_to_next <- FALSE
     
-    tryCatch({
-      
+    message(paste0("Analysis number: ",sens))
+
     start_time_analysis <-  proc.time()
     
     output_sim[[sens]] <- list() #initialize analysis lists
@@ -366,11 +388,7 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
                        debug = debug,
                        accum_backwards = accum_backwards,
                        continue_on_error = continue_on_error,
-                       log_list = list(),
-                       env_setup_sens = env_setup_sens,
-                       env_setup_sim = env_setup_sim,
-                       env_setup_pt = env_setup_pt,
-                       env_setup_arm = env_setup_arm
+                       log_list = list()
                       )
     
     if(is.null(seed)){
@@ -381,20 +399,15 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
     # Draw Common parameters  -------------------------------
     input_list_sens <- as.environment(input_list_sens)
     parent.env(input_list_sens) <- environment()
+    .set_last_ctx(stage="Error in setup:sensitivity_inputs", sens=sens, .warden_ctx = .warden_ctx)
     
     # Draw Common parameters  -------------------------------
     if(!is.null(sensitivity_inputs)){
       
-      if(env_setup_sens){
-        load_inputs2(inputs = input_list_sens,list_uneval_inputs = sensitivity_inputs)
-      } else{
-        input_list_sens <- as.environment(
-          load_inputs(inputs = as.list(input_list_sens),
-                      list_uneval_inputs = sensitivity_inputs)
-        )
-        parent.env(input_list_sens) <- environment()
-        
-      }
+      on_error_check({
+          load_inputs(inputs = input_list_sens,list_uneval_inputs = sensitivity_inputs)
+      }, continue_on_error = continue_on_error)
+      if(.skip_to_next){next}
       
       if(input_list_sens$debug){ 
         dump_info <- debug_inputs(NULL,input_list_sens)
@@ -405,6 +418,8 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
         )
         
         log_list <- c(log_list,dump_info)
+        log_add(dump_info)
+        
       }
       
     }
@@ -413,7 +428,8 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
 # Simulation loop ---------------------------------------------------------
     output_sim[[sens]] <- vector("list", length=n_sim) # empty list with n_sim elements
     # Outer loop, repeat for each patient
-    exported_items <- unique(c("input_list_sens",ls(.GlobalEnv),ls(parent.env(environment())),ls(environment())))
+    
+    exported_items <- unique(c("input_list_sens",ls(.GlobalEnv),ls(parent.env(environment()), all.names = TRUE),ls(environment(), all.names = TRUE)))
     options(future.rng.onMisuse = "ignore")
     
     output_sim[[sens]] <- foreach(simulation = 1:n_sim,
@@ -421,34 +437,31 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
                          .options.future = list(packages = .packages()),
                          .options.future = list(globals=structure(TRUE,add = exported_items)),
                          .combine = 'c') %dofuture% {
+      .set_last_ctx(stage="Error in setup:simulation_start", sens=sens, simulation=simulation, .warden_ctx = .warden_ctx)
                            
       RNGkind("L'Ecuyer-CMRG") #repeat this here so parallel::RngStream does not malfunction
                             
       message(paste0("Simulation number: ",simulation))
       
-      tryCatch({
-        
                            
       start_time_sim <-  proc.time()
                            
                            
-      input_list <- rlang::env_clone(input_list_sens , parent.env(input_list_sens))
+      # input_list <- rlang::env_clone(input_list_sens , parent.env(input_list_sens))
+      input_list <- new.env(parent = input_list_sens)
+      list2env(as.list(input_list_sens), input_list) 
       input_list$simulation <- simulation
       
       set.seed(simulation*1007*seed)
+      .set_last_ctx(stage="setup:common_all_inputs", sens=sens, simulation=simulation, .warden_ctx = .warden_ctx)
+      
       
       # Draw Common parameters  -------------------------------
       if(!is.null(common_all_inputs)){
-        
-        if(env_setup_sim){
-          load_inputs2(inputs = input_list,list_uneval_inputs = common_all_inputs)
-        } else{
-          input_list <- as.environment(
-            load_inputs(inputs = as.list(input_list),
-                        list_uneval_inputs = common_all_inputs)
-          )
-          parent.env(input_list) <- parent.env(input_list_sens)
-        }
+        on_error_check({
+              load_inputs(inputs = input_list,list_uneval_inputs = common_all_inputs)
+        }, continue_on_error = continue_on_error)
+          if(.skip_to_next){return(NULL)}
         
         if(input_list_sens$debug){ 
           dump_info <- debug_inputs(input_list_sens,input_list)
@@ -460,39 +473,37 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
           )
           
           log_list <- c(log_list,dump_info)
+          log_add(dump_info)
         }
-      }
+        }
+      
   
       if(is.null(input_list$drc)){input_list$drc <- 0.03}
       if(is.null(input_list$drq)){input_list$drq <- 0.03}
       
       # Run engine ----------------------------------------------------------
-        final_output <- run_engine(arm_list=arm_list,
-                                        common_pt_inputs=common_pt_inputs,
-                                        unique_pt_inputs=unique_pt_inputs,
-                                        input_list = input_list,
-                                        pb = pb,
-                                        seed = seed)                    
-      
-      if(!is.null(final_output$error_m)){
-        if((n_sim > 1 | n_sensitivity > 1) & continue_on_error){
-          message(if(debug){"Log will be exported. "},
-                  "Continuing on error, error message at analysis ",
-                  sens,
-                  "; simulation: ",
-                  if(exists("simulation")){simulation}else{"None"},
-                  ". Error message: ",final_output$error_m
-          )
-          if(debug){
-            return(list(final_output))
+        .set_last_ctx(stage="Error in engine:start", sens=sens, simulation=simulation, .warden_ctx = .warden_ctx)
+        on_error_check({
+          if(constrained){
+            final_output <- run_engine_constrained(arm_list=arm_list,
+                                                   common_pt_inputs=common_pt_inputs,
+                                                   unique_pt_inputs=unique_pt_inputs,
+                                                   input_list = input_list,
+                                                   pb = pb,
+                                                   seed = seed,
+                                                   .warden_ctx = .warden_ctx)  
+          } else{
+            final_output <- run_engine(arm_list=arm_list,
+                                       common_pt_inputs=common_pt_inputs,
+                                       unique_pt_inputs=unique_pt_inputs,
+                                       input_list = input_list,
+                                       pb = pb,
+                                       seed = seed,
+                                       .warden_ctx = .warden_ctx)   
           }
-          
-          return(list(NULL))
-        } else{
-          stop(final_output$error_m)
-        }
-      }
-  
+        }, continue_on_error = continue_on_error)
+        if(.skip_to_next){return(NULL)}
+      
       if (input_list$ipd>0) {
   
         final_output$merged_df$simulation <- simulation
@@ -501,135 +512,31 @@ run_sim_parallel <- function(arm_list=c("int","noint"),
       
       final_output <- c(list(sensitivity_name = sens_name_used), final_output)
       
-      if(input_list$debug){
-        log_list <- lapply(log_list,transform_debug)
-        
-        final_output$log_list <- c(log_list,final_output$log_list)
-      }
+      if (debug) final_output$log_list <- log_sink$entries
       
       return(list(final_output))
       
       message(paste0("Time to run simulation ", simulation,": ",  round(proc.time()[3]- start_time_sim[3] , 2 ), "s"))
       
-      }, error = function(e) {
-        if(continue_on_error){
-          .skip_to_next <<- TRUE
-          message(if(debug){"Log will be exported. "},
-                  "Continuing on error, error message at analysis ",
-                  sens,
-                  "; simulation: ",
-                  if(exists("simulation")){simulation}else{"None"},
-                  ". Error message: ",e$message," in ", e$call)
-        } else{
-          if(debug){
-            stop("Due to using a parallel engine for simulations, debug file will only include analysis inputs if continue_on_error = FALSE.
-                 Error message at analysis ",
-                  sens,
-                  "; simulation: ",
-                  if(exists("simulation")){simulation}else{"None"},
-                  ". Error message: ",
-                  e$message," in ", e$call)
-          }else{
-            stop("Error message at analysis ",
-                 sens,
-                 "; simulation: ",
-                 if(exists("simulation")){simulation}else{"None"},
-                 ". Error message: ",
-                 e$message," in ", e$call)
-          }
-        }
-      }
-      )
-      if(.skip_to_next) { return(NULL) } 
-      
+
      }
     
 
     message(paste0("Time to run analysis ", sens,": ",  round(proc.time()[3]- start_time_analysis[3] , 2 ), "s"))
     
-    }, error = function(e) {
-      if(continue_on_error){
-        .skip_to_next <<- TRUE
-        message(if(debug){"Log will be exported. "},
-                "Continuing on error, error message at analysis ",
-                sens,
-                "; simulation: ",
-                if(exists("simulation")){simulation}else{"None"},
-                ". Error message: ",
-                e$message," in ", e$call)
-      } else{
-        if(debug){
-          if(!exists("final_output") & length(log_list)>0){
-            export_log(lapply(log_list,transform_debug),paste0("log_model_",format(Sys.time(), "%Y_%m_%d_%Hh_%mm_%Ss"),".txt"))
-            stop(e$message)
-          }else if(exists("output_sim")){
-            final_log <- unlist(
-              unlist(
-                lapply(output_sim, function(y) lapply(y, function(x) x$log_list )),
-                recursive = FALSE),
-              recursive = FALSE)
-            
-            export_log(final_log,paste0("log_model_",format(Sys.time(), "%Y_%m_%d_%Hh_%mm_%Ss"),".txt"))
-            
-          } else if(!exists("output_sim") & exists("final_output")){
-            log_list <- lapply(log_list,transform_debug)
-            final_output <- list()
-            final_output$log_list <- c(log_list,final_output$log_list)
-            final_log <- final_output$log_list
-            
-            export_log(final_log,paste0("log_model_",format(Sys.time(), "%Y_%m_%d_%Hh_%mm_%Ss"),".txt"))
-            
-          } else{
-            message("No data to export.")
-          }
-          
-          stop("Log will be exported if data exists. Error message at analysis ",
-               sens,
-               "; simulation: ",
-               if(exists("simulation")){simulation}else{"None"},
-               ". Error message: ",
-               e$message," in ", e$call)
-        }else{
-          stop(e$message)
-        }
-      }
-    }
-        )
-          if(.skip_to_next) { next } 
-      
+
   }
   message(paste0("Total time to run: ",  round(proc.time()[3]- start_time[3] , 2), "s"))
-  
-
-  # Export results ----------------------------------------------------------
-  if(debug){
-    if(length(log_list)>0 | exists("output_sim")){
-        if(exists("output_sim")){
-          final_log <- unlist(
-            unlist(
-              lapply(output_sim, function(y) lapply(y, function(x) x$log_list )),
-              recursive = FALSE),
-            recursive = FALSE)
-        } else{
-          log_list <- lapply(log_list,transform_debug)
-          final_output <- list()
-          final_output$log_list <- c(log_list,final_output$log_list)
-          final_log <- final_output$log_list
-        }
-        
-        export_log(final_log,paste0("log_model_",format(Sys.time(), "%Y_%m_%d_%Hh_%mm_%Ss"),".txt"))
-    } else{
-        message("No data to export.")
-      }
-    }
+ 
   
   
 
   results <- output_sim
+    
+  }, enable=TRUE, cleanup = TRUE) 
   
-  RNGkind(rng_kind_store)
+  .set_last_ctx(stage="Simulation finalized", .warden_ctx = .warden_ctx)
   
-  }, enable=TRUE) 
   
   return(results)
 
